/*
 *   Copyright (c) 2000 S.Noda <sakae_noda@hotmail.com>
 *
 *   Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted provided
 * that the above copyright notice and this permission notice appear
 * in all copies of the software and related documentation.
 *
 *                          NO WARRANTY
 *
 *   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES;
 * WITHOUT EVEN THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

#include "stdafx.h"
#include "zp.h"
#include "StatusDlg.h"
#include "PasswordDlg.h"
#include "CommentDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define Z_DLL_NAME "ZIP32.DLL\0"

static CStatusDlg* pDlg;
static BOOL bDisplayDialog;

void CreateStatusDlg(BOOL bDisplayDialog)
{
	::bDisplayDialog = bDisplayDialog;
	if (bDisplayDialog)
	{
		pDlg = new CStatusDlg();
		pDlg->Create();
		pDlg->m_ctrlOk.EnableWindow(FALSE);
		pDlg->m_ctrlStatus.EnableWindow(FALSE);
		pDlg->m_bCancel = FALSE;
	}
}

void DestroyStatusDlg()
{
	if (bDisplayDialog)
	{
		pDlg->UpdateData();
		pDlg->m_ctrlStatus.EnableWindow();
		pDlg->m_ctrlOk.EnableWindow();
		pDlg->m_ctrlOk.SetFocus();
		
		if (pDlg->m_bClose)
		{
			pDlg->DestroyWindow();
			pDlg = NULL;
		}
	}
}

void DoEvents()
{
	MSG msg;
	while (PeekMessage(&msg, (HWND)NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

int WINAPI ZpPrnt(LPSTR buf, unsigned long size)
{
	if (bDisplayDialog)
	{
		pDlg->m_ctrlStatus.SetSel(-1,-1);
		pDlg->m_ctrlStatus.ReplaceSel(buf);
	}
	else
		TRACE("%s", buf);
	DoEvents();
	
	return (unsigned int) size;
}

int WINAPI ZpPassword(LPSTR p, int n, LPCSTR m, LPCSTR name)
{
	CPasswordDlg dlg;
	
	dlg.m_stPrompt = m;
	
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
		lstrcpy(p, dlg.m_stPassword);
	else if (nResponse == IDCANCEL)
		lstrcpy(p, "\0");
	
	return 0;
}

int WINAPI ZpService(LPCSTR filename, unsigned long size)
{
	TRACE("<ZpService: %s (%d)>\n", filename, size);
	if (bDisplayDialog)
		return pDlg->m_bCancel;
	return FALSE;
}

int WINAPI ZpComment(LPSTR buf)
{
	CCommentDlg dlg;
	
	dlg.m_stPrompt = "enter new zip file comment: ";
	dlg.m_stComment = buf;
	
	int nResponse = dlg.DoModal();
	
	if (nResponse == IDOK)
		lstrcpy(buf, dlg.m_stComment);
	else if (nResponse == IDCANCEL)
		lstrcpy(buf, "\0");
	
	return TRUE;
}

int ZpMain(ZCL CL, LPZPOPT lpOPT)
{
	int ret;
	
	HANDLE hUF = GlobalAlloc(GPTR, (DWORD)sizeof(ZIPUSERFUNCTIONS));
	if (!hUF)
	{
		return -11;
	}
	LPZIPUSERFUNCTIONS lpUserFunc = (LPZIPUSERFUNCTIONS)GlobalLock(hUF);
	if (!lpUserFunc)
	{
		GlobalFree(hUF);
		return -12;
	}
	
	lpUserFunc->print = ZpPrnt;
	lpUserFunc->comment = ZpComment;
	lpUserFunc->password = ZpPassword;
	lpUserFunc->ServiceApplication = ZpService;
	
	HINSTANCE hZipDll = LoadLibrary(Z_DLL_NAME);
	if (!hZipDll)
	{
		GlobalUnlock(hUF);
		GlobalFree(hUF);
		return -13;
	}
	ZP_VERSION fpZpVersion = (ZP_VERSION)GetProcAddress(hZipDll, "ZpVersion");
	ZP_INIT fpZpInit = (ZP_INIT)GetProcAddress(hZipDll, "ZpInit");
	ZP_SETOPTIONS fpZpSetOptions = (ZP_SETOPTIONS)GetProcAddress(hZipDll, "ZpSetOptions");
	ZP_ARCHIVE fpZpArchive = (ZP_ARCHIVE)GetProcAddress(hZipDll, "ZpArchive");
	if (!fpZpVersion || !fpZpInit || !fpZpSetOptions || !fpZpArchive)
	{
		GlobalUnlock(hUF);
		GlobalFree(hUF);
		return -14;
	}
	
	ZpVer ver;
	(*fpZpVersion)(&ver);
	_RPTF1(_CRT_WARN, "<DLL Date: %s>\n", ver.date);
	_RPTF3(_CRT_WARN, "<DLL Version: %d.%d%d>\n", ver.windll.major, ver.windll.minor, ver.windll.patchlevel);
	if (ver.windll.major == 2 && ver.windll.minor == 3)
	{
		if ((*fpZpInit)(lpUserFunc))
		{
			CreateStatusDlg(!lpOPT->fQuiet);
			(*fpZpSetOptions)(lpOPT);
			ret = (*fpZpArchive)(CL);
			DestroyStatusDlg();
		} else ret = -16;
	} else ret = -15;
	
	GlobalUnlock(hUF);
	GlobalFree(hUF);
	FreeLibrary(hZipDll);
	return ret; // if (ret != 0) then "Error"
	
}
